
------------------------------------------------------------------------
-- StatBlockCore, a core for informational blocks
------------------------------------------------------------------------

local SBC = CreateFrame("Frame", "StatBlockCore")
local L = LibStub("AceLocale-3.0"):GetLocale("StatBlockCore", true)

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local cbh = LibStub:GetLibrary("CallbackHandler-1.0"):New(SBC)
local media = LibStub("LibSharedMedia-3.0")
local SBC_block = {}
local SBC_display = {}
local SBC_icon = {}
local db

------------------------------
--           Config         --
------------------------------

local options
local function getOptions()
	if not options then
		options = {
			name = "StatBlocksCore",
			childGroups = "tab", type = "group",
			args = {
				main = {
					name = "Core",
					order = 1, type = "group",
					args = {
						desc = {
							order = 1,
							name = L["A core for your informational blocks"],
							type = "description",
						},
						spacer = {
							name = "",
							order = 2, type = "header",
						},
						alldesc = {
							order = 3, type = "description",
							name = L["Change global block settings"],
						},
						globalvaluecolor = {
							order = 4, type = "color",
							name = L["Global Value Color"],
							get = function() return db.globalColor.r, db.globalColor.g, db.globalColor.b end,
							set = function(_, r, g, b)
								db.globalColor.r = r db.globalColor.g = g db.globalColor.b = b
								for k,v in pairs(SBC_display) do
									db.plugins[k].color.r = r db.plugins[k].color.g = g db.plugins[k].color.b = b
									v:SetTextColor(r,g,b)
								end
							end,
						},
						globallabelcolor = {
							order = 5, type = "color",
							name = L["Global Label Color"],
							get = function() return db.globalLabelColor.r, db.globalLabelColor.g, db.globalLabelColor.b end,
							set = function(_, r, g, b)
								db.globalLabelColor.r = r db.globalLabelColor.g = g db.globalLabelColor.b = b
								local c = ("%02x%02x%02x"):format(r*255, g*255, b*255)
								for k in pairs(SBC_block) do
									local obj = ldb:GetDataObjectByName(k)
									if obj.label and not db.plugins[k].labelc then
										db.plugins[k].labelc = c
									end
									if db.plugins[k].labelc then
										db.plugins[k].labelc = c
										db.plugins[k].labelColor.r = r db.plugins[k].labelColor.g = g db.plugins[k].labelColor.b = b
										SBC:TextUpdate(nil, k, nil, obj.text, obj)
									end
								end
							end,
						},
						globalfont = {
							order = 6, type = "select", width = "double",
							name = L["Global Font"],
							values = media:List("font"),
							get = function()
								for k,v in pairs(media:List("font")) do
									if db.globalFont == v then
										return k
									end
								end
							end,
							set = function(_, font)
								db.globalFont = media:List("font")[font]
								SBC:UpdateFont()
							end,
						},
						globalalpha = {
							order = 7, type = "range", width = "double",
							name = L["Global Backdrop Alpha"],
							min = 0, max = 1, step = 0.01,
							get = function() return db.globalAlpha end,
							set = function(_, a)
								db.globalAlpha = a
								for _,v in pairs(SBC_block) do
									v:SetBackdropColor(db.globalBDC.r,db.globalBDC.g,db.globalBDC.b,a)
								end
							end,
						},
						globalsize = {
							order = 8, type = "range", width = "double",
							name = L["Global Font Size"],
							min = 1, max = 48, step = 1,
							get = function() return db.globalSize end,
							set = function(_, size)
								db.globalSize = size
								for _,v in pairs(SBC_display) do
									v:SetFont(media:Fetch("font", db.globalFont), size)
								end
							end,
						},
						globallock = {
							order = 10, type = "toggle",
							name = L["Global Lock"],
							get = function() return db.globalLock end,
							set = function(_, state)
								db.globalLock = state
								for k in pairs(SBC_block) do
									db.plugins[k].lock = state
									local l
									if not state then l = true else l = false end
									SBC_block[k]:SetMovable(l)
									SBC_block[k]:EnableMouseWheel(l)
								end
							end,
						},
						globalbgcolor = {
							order = 11, type = "color",
							name = L["Global Backdrop Color"],
							get = function() return db.globalBDC.r, db.globalBDC.g, db.globalBDC.b end,
							set = function(_, r, g, b)
								db.globalBDC.r = r db.globalBDC.g = g db.globalBDC.b = b
								for _,v in pairs(SBC_block) do
									v:SetBackdropColor(r,g,b)
								end
							end,
						},
					},
				},
				profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(SBC.db)
			},
		}
	end
	return options
end

local blockOpts = {}
local function getBlockOptions(name)
	if not blockOpts[name] then
		local obj = ldb:GetDataObjectByName(name)
		blockOpts[name] = {
			name = name,
			order = 1, type = "group",
			args = {
				lock = {
					name = _G["LOCK"],
					order = 10, type = "toggle", width = "half",
					get = function() return db.plugins[name].lock end,
					set = function(_, state) db.plugins[name].lock = state
						if not state then state = true else state = false end
						SBC_block[name]:SetMovable(state)
						SBC_block[name]:EnableMouseWheel(state)
					end,
				},
				color = {
					name = L["Value Color"],
					order = 20, type = "color", width = "half",
					get = function() return db.plugins[name].color.r, db.plugins[name].color.g, db.plugins[name].color.b end,
					set = function(_, r, g, b) local cdb = db.plugins[name].color 
						cdb.r = r cdb.g = g cdb.b = b SBC_display[name]:SetTextColor(r,g,b)
					end,
				},
				labelcolor = {
					name = L["Label Color"],
					order = 30, type = "color", width = "half",
					get = function() return db.plugins[name].labelColor.r, db.plugins[name].labelColor.g, db.plugins[name].labelColor.b end,
					set = function(_, r, g, b)
						local c = ("%02x%02x%02x"):format(r*255, g*255, b*255)
						if not db.plugins[name].labelc then
							db.plugins[name].labelc = c
							db.plugins[name].labelColor = {r=1,g=1,b=1}
						end
						db.plugins[name].labelc = c
						db.plugins[name].labelColor.r = r db.plugins[name].labelColor.g = g db.plugins[name].labelColor.b = b
						SBC:TextUpdate(nil, name, nil, obj.text, obj)
					end,
					disabled = function() return not obj.label end,
				},
				label = {
					name = L["Text Display"],
					order = 40, type = "input",
					get = function() return db.plugins[name].label or obj.label or "N/A" end,
					set = function(_, label)
						obj.label = label
						db.plugins[name].label = label
						SBC:TextUpdate(nil, name, nil, obj.text, obj)
					end,
					disabled = function() return not obj.label end,
				},
				align = {
					name = L["Font Align"],
					order = 50, type = "select", 
					values = {LEFT = L["LEFT"], CENTER = L["CENTER"], RIGHT = L["RIGHT"]},
					get = function() return db.plugins[name].align end,
					set = function(_, e) db.plugins[name].align = e SBC_display[name]:SetJustifyH(e) end,
				},
				x = {
					name = L["X Position"],
					order = 60, type = "input",
					get = function() return tostring(db.plugins[name].posx / SBC_block[name]:GetEffectiveScale()) end,
					set = function(_, x) x = tonumber(x)
						local s = SBC_block[name]:GetEffectiveScale()
						db.plugins[name].posx = x * s
						SBC_block[name]:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, db.plugins[name].posy / s)
					end,
				},
				y = {
					name = L["Y Position"],
					order = 70, type = "input",
					get = function() return tostring(db.plugins[name].posy / SBC_block[name]:GetEffectiveScale()) end,
					set = function(_, y) y = tonumber(y)
						local s = SBC_block[name]:GetEffectiveScale()
						db.plugins[name].posy = y * s
						SBC_block[name]:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.plugins[name].posx / s, y)
					end,
				},
				scale = {
					name = L["Scale"],
					order = 80, type = "range", width = "full",
					min = 0.5, max = 2, step = 0.01,
					get = function() return db.plugins[name].scale end,
					set = function(_, scale) db.plugins[name].scale = scale SBC_block[name]:SetScale(scale) end,
				},
				parent = {
					name = L["Parenting"],
					order = 90, type = "input",
					get = function() return db.plugins[name].parent end,
					set = function(_, p)
						db.plugins[name].parent = p
						SBC_block[name]:SetParent(p)
					end,
				},
				strata = {
					name = L["Strata"],
					order = 100, type = "select",
					get = function() return db.plugins[name].strata end,
					set = function(_, strata) db.plugins[name].strata = strata
						SBC_block[name]:SetFrameStrata(db.plugins[name].strata)
					end,
					values = {HIGH = _G["HIGH"], MEDIUM = _G["AUCTION_TIME_LEFT2"], LOW = _G["LOW"], BACKGROUND = _G["BACKGROUND"]},
				},
				--[[icon = {
					name = _G["EMBLEM_SYMBOL"], --Icon
					order = 100, type = "toggle",
					get = function() return db.plugins[name].icon end,
					set = function(_, i) db.plugins[name].icon = i
						if i then SBC_icon[name]:Show() else SBC_icon[name]:Show() end
					end,
				},]]
			},
		}
	end
	return blockOpts[name]
end

------------------------------
--      Main Functions      --
------------------------------

SBC:RegisterEvent("ADDON_LOADED")
SBC:RegisterEvent("PLAYER_LOGIN")
SBC:SetScript("OnEvent", function(_, event, ...) SBC[event](SBC, ...) end)

local txtfmt = "%s|cff%s%s|r"
local defaultc = "ffffff"
function SBC:TextUpdate(_, name, _, data, obj)
	if obj.label then
		SBC_display[name]:SetFormattedText(txtfmt, data, db.plugins[name].labelc or defaultc, obj.label)
	else
		SBC_display[name]:SetText(data)
	end
end

function SBC:Tooltip(_,name,_,func)
	SBC_block[name]:SetScript("OnEnter", function()
		if not this.isMoving then
			GameTooltip:SetOwner(SBC_block[name], "ANCHOR_CURSOR")
			func(SBC_block[name])
			GameTooltip:Show()
		end
	end)
	SBC_block[name]:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

function SBC:Click(_,name,_,func)
	SBC_block[name]:SetScript("OnMouseUp", function(_, btn) if not this.isMoving then func(SBC_block[name], btn) end end)
end

--[[function SBC:Icon(_,name)
	local t = SBC_block[name]:CreateTexture(nil,"BACKGROUND")
t:SetTexture("")
t:SetAllPoints(f)


end]]

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function SBC:New(_, name)
	local obj = ldb:GetDataObjectByName(name)
	if obj.launcher then return end
	if db.plugins[name].label then
		obj.label = db.plugins[name].label
	end

	_G.LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("StatBlock_"..name, getBlockOptions(name))
	_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StatBlock_"..name, name, "StatBlockCore")

	SBC_block[name] = CreateFrame("Frame", "StatBlockCore_"..name, UIParent)
	SBC_block[name]:SetBackdrop(bg)
	SBC_block[name]:SetBackdropColor(db.globalBDC.r,db.globalBDC.g,db.globalBDC.b,db.globalAlpha)
	SBC_block[name]:EnableMouse(true)
	if not db.plugins[name].lock then SBC_block[name]:SetMovable(true) SBC_block[name]:EnableMouseWheel(true) end
	SBC_block[name]:SetFrameStrata(db.plugins[name].strata)
	SBC_block[name]:RegisterForDrag("LeftButton")
	SBC_block[name]:SetScript("OnDragStart", function() if this:IsMovable() then this.isMoving = true this:StartMoving() GameTooltip:Hide() end end)
	SBC_block[name]:SetScript("OnMouseWheel", function(_, d)
		if db.plugins[name].lock then return end
		local w = SBC_block[name]:GetWidth() + d
		SBC_block[name]:SetWidth(w) SBC_display[name]:SetWidth(w) db.plugins[name].width = w
	end)
	SBC_block[name]:SetScript("OnDragStop", function() this:StopMovingOrSizing()
		--try to stick to a screen edge, then try to stick to a bar [ Thank you Tuller <3 ]
		for _,v in pairs(SBC_block) do
			if FlyPaper.Stick(this, v, 20, 2, 2) then
				break
			end
		end
		for k,v in pairs(SBC_block) do
			local s = v:GetEffectiveScale()
			db.plugins[k].posx = v:GetLeft() * s db.plugins[k].posy = v:GetTop() * s
		end
		this.isMoving = nil
	end)
	SBC_block[name]:SetWidth(db.plugins[name].width)
	SBC_block[name]:SetHeight(20)
	SBC_block[name]:Show()
	SBC_block[name]:SetScale(db.plugins[name].scale)

	if db.plugins[name].posx == 0 and db.plugins[name].posy == 0 then
		SBC_block[name]:ClearAllPoints()
		SBC_block[name]:SetPoint("CENTER", UIParent, "CENTER")
	else
		local s = SBC_block[name]:GetEffectiveScale()
		SBC_block[name]:ClearAllPoints()
		SBC_block[name]:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.plugins[name].posx / s, db.plugins[name].posy / s)
	end
	SBC_block[name]:SetParent(db.plugins[name].parent)

	SBC_display[name] = SBC_block[name]:CreateFontString("StatBlockCoreFont_"..name, "OVERLAY")
	SBC_display[name]:SetPoint("CENTER", SBC_block[name], "CENTER")
	SBC_display[name]:SetFont(media:Fetch("font", db.globalFont), db.globalSize)
	SBC_display[name]:SetTextColor(db.plugins[name].color.r, db.plugins[name].color.g, db.plugins[name].color.b)
	SBC_display[name]:SetWidth(db.plugins[name].width)
	SBC_display[name]:SetJustifyH(db.plugins[name].align)
	SBC_display[name]:SetShadowColor(0, 0, 0, 1)
	SBC_display[name]:SetShadowOffset(1, -1)

	ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_text", "TextUpdate")
	ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_OnEnter", "Tooltip")
	ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_OnClick", "Click")
end

function SBC:ADDON_LOADED(msg)
	if msg == "StatBlockCore" then
		local defaults = {
			profile = {
				globalFont = "Friz Quadrata TT",
				globalSize = 20,
				globalColor = { r = 1, g = 1, b = 1 },
				globalBDC = { r = 1, g = 1, b = 1 },
				globalAlpha = 1,
				globalLock = nil,
				globalLabelColor = { r = 1, g = 1, b = 1 },
				plugins = {
					["*"] = {
						color = { r = 1, g = 1, b = 1, },
						lock = nil,
						labelColor = { r = 1, g = 1, b = 1 },
						scale = 1,
						align = "CENTER",
						width = 100,
						posx = 0,
						posy = 0,
						parent = "UIParent",
						strata = "BACKGROUND",
						icon = true,
					},
				},
			},
		}
		self:UnregisterEvent("ADDON_LOADED")
		self.db = _G.LibStub("AceDB-3.0"):New("StatBlockCoreDB", defaults)
		db = self.db.profile
		ldb.RegisterCallback(self, "LibDataBroker_DataObjectCreated", "New")

		local txt = "|cff33ff99StatBlocks|r: "
		_G.LibStub("AceConfig-3.0"):RegisterOptionsTable("StatBlockCore", getOptions)
		_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StatBlockCore")
		_G.SlashCmdList["STATBLOCKS_MAIN"] = function()
			InterfaceOptionsFrame_OpenToFrame("StatBlockCore")
			_G.ChatFrame1:AddMessage(txt.. L["Open_Alert"])
		end
		_G.SLASH_STATBLOCKS_MAIN1 = "/sbc"
		_G.SLASH_STATBLOCKS_MAIN2 = "/statblockcore"

		media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateFont")
		media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateFont")
		local function profilePrint() _G.ChatFrame1:AddMessage(txt.. L["Reload UI"]) end
		self.db.RegisterCallback(self, "OnProfileReset", profilePrint)
		self.db.RegisterCallback(self, "OnProfileChanged", profilePrint)
		SBC.ADDON_LOADED = nil
	end
end

function SBC:UpdateFont()
	local font = media:Fetch("font", db.globalFont)
	for _,v in pairs(SBC_display) do
		v:SetFont(font, db.globalSize)
	end
end

function SBC:PLAYER_LOGIN()
	for k,v in pairs (SBC_block) do
		if db.plugins[k].posx == 0 and db.plugins[k].posy == 0 then
			v:ClearAllPoints()
			v:SetPoint("CENTER", UIParent, "CENTER")
		else
			local s = v:GetEffectiveScale()
			v:ClearAllPoints()
			v:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.plugins[k].posx / s, db.plugins[k].posy / s)
		end
	end
	SBC:SetScript("OnEvent", nil)
	SBC.PLAYER_LOGIN = nil
end

------------------------------
--         Cleanup          --
------------------------------

local prev = 0
SBC:SetScript("OnUpdate", function(_, c)
	prev = prev + c
	if prev > 7 then
		SBC:SetScript("OnUpdate", nil)
		ldb.UnregisterCallback(SBC, "LibDataBroker_DataObjectCreated")
		SBC.New = nil
		SBC.Click = nil
		SBC.Tooltip = nil
		prev = nil
		for k in pairs(SBC_block) do
			ldb.UnregisterCallback(SBC, "LibDataBroker_AttributeChanged_"..k.."_OnClick")
			ldb.UnregisterCallback(SBC, "LibDataBroker_AttributeChanged_"..k.."_OnEnter")
		end
	end
end)

